@@ -18,7 +18,7 @@ class CrowdClient:
         self.session = requests.session()
         self.session.verify = verify_cert
 
-    def authenticate(self) -> CrowdClient:
+    def authenticate(self) -> self:
         """
         Authenticate to CrowdStrike API using id and secret supplied on instantiation
         :return:
@@ -413,7 +413,7 @@ class RTRClient:
         self.session = requests.session()
         self.session.verify = verify_cert
 
-    def authenticate(self) -> RTRClient:
+    def authenticate(self) -> self:
         """
         Authenticate to CrowdStrike API using id and secret supplied on instantiation.
         :return:
@@ -510,7 +510,7 @@ class RTRClient:
         :param batch_id: Batch ID retrieved from the batch_init() method.
         :param command: Valid commands: cat, cd, clear, env, eventlog, filehash, getsid, help, history, ipconfig, ls,
                         mount, netstat, ps, reg query.
-        :param command_string: Full command string for the command - IE, 'cd C:\some_directory'.
+        :param command_string: Full command string for the command - IE, 'cd C:\\some_directory'.
         :param timeout: How long to wait for the request in seconds - Default is 30, max is 10 minutes (600).
         :param timeout_duration: How long to wait for the request in duration syntax. Valid units: ns, us, ms, s, m, h.
         :param optional_hosts: List of a subset of hosts to run a command on.
@@ -543,7 +543,7 @@ class RTRClient:
         :param command: Valid commands: cat, cd, clear, cp, encrypt, env, eventlog, filehash, get, getsid, help,
                         history, ipconfig, kill, ls, map, memdump, mkdir, mount, mv, netstat, ps, reg query, reg set,
                         reg delete, reg load, reg unload, restart, rm, runscript, shutdown, unmap, xmemdump, zip.
-        :param command_string: Full command string for the command - IE, 'cd C:\some_directory'.
+        :param command_string: Full command string for the command - IE, 'cd C:\\some_directory'.
         :param timeout: How long to wait for the request in seconds - Default is 30, max is 10 minutes (600).
         :param timeout_duration: How long to wait for the request in duration syntax. Valid units: ns, us, ms, s, m, h.
         :param optional_hosts: List of a subset of hosts to run a command on.
@@ -578,7 +578,7 @@ class RTRClient:
                         history, ipconfig, kill, ls, map, memdump, mkdir, mount, mv, netstat, ps, put, reg query,
                         reg set, reg delete, reg load, reg unload, restart, rm, run, runscript, shutdown, unmap,
                         xmemdump, zip.
-        :param command_string: Full command string for the command - IE, 'cd C:\some_directory'.
+        :param command_string: Full command string for the command - IE, 'cd C:\\some_directory'.
         :param timeout: How long to wait for the request in seconds - Default is 30, max is 10 minutes (600).
         :param timeout_duration: How long to wait for the request in duration syntax. Valid units: ns, us, ms, s, m, h.
         :param optional_hosts: List of a subset of hosts to run a command on.
@@ -684,7 +684,7 @@ class RTRClient:
         :param session_id: Session ID retrieved from the batch_init() method.
         :param command: Valid commands: cat, cd, clear, env, eventlog, filehash, getsid, help, history, ipconfig, ls,
                         mount, netstat, ps, reg query.
-        :param command_string: Full command string for the command - IE, 'cd C:\some_directory'.
+        :param command_string: Full command string for the command - IE, 'cd C:\\some_directory'.
         :return:
         """
 
@@ -701,11 +701,11 @@ class RTRClient:
     def session_active_cmd(self, session_id: str, command: str, command_string: str) -> bool:
         """
         Executes an RTR active-responder command across all hosts mapped to a batch ID.
-        :param session_id: Batch ID retrieved from the batch_init() method.
+        :param session_id: Session ID retrieved from the batch_init() method.
         :param command: Valid commands: cat, cd, clear, cp, encrypt, env, eventlog, filehash, get, getsid, help,
                         history, ipconfig, kill, ls, map, memdump, mkdir, mount, mv, netstat, ps, reg query, reg set,
                         reg delete, reg load, reg unload, restart, rm, runscript, shutdown, unmap, xmemdump, zip.
-        :param command_string: Full command string for the command - IE, 'cd C:\some_directory'.
+        :param command_string: Full command string for the command - IE, 'cd C:\\some_directory'.
         :return:
         """
 
@@ -722,12 +722,12 @@ class RTRClient:
     def session_admin_cmd(self, session_id: str, command: str, command_string: str) -> bool:
         """
         Executes an RTR read-only command across all hosts mapped to a batch ID.
-        :param session_id: Batch ID retrieved from the batch_init() method.
+        :param session_id: Session ID retrieved from the batch_init() method.
         :param command: Valid commands: cat, cd, clear, cp, encrypt, env, eventlog, filehash, get, getsid, help,
                         history, ipconfig, kill, ls, map, memdump, mkdir, mount, mv, netstat, ps, put, reg query,
                         reg set, reg delete, reg load, reg unload, restart, rm, run, runscript, shutdown, unmap,
                         xmemdump, zip.
-        :param command_string: Full command string for the command - IE, 'cd C:\some_directory'.
+        :param command_string: Full command string for the command - IE, 'cd C:\\some_directory'.
         :return:
         """
 
@@ -741,6 +741,34 @@ class RTRClient:
 
         return response.status_code == 201
 
+    def session_runscript(self, session_id: str, script_name: str = None, raw_cmd: str = None) -> bool:
+        """
+        Execute either a pre-staged PowerShell script or supply a raw PowerShell command. Added this specifically
+        because the format for the runscript command can be tricky and is a pain to debug.
+        :param session_id: Session ID retrieved from the batch_init() method.
+        :param script_name: Name of a pre-staged script.
+        :param raw_cmd: Valid PowerShell command to execute across hosts in batch session.
+        :return:
+        """
+
+        if script_name:
+            cmd_string = f'runscript -CloudFile="{script_name}"'
+        elif raw_cmd:
+            cmd_string = f'runscript -Raw=```{raw_cmd}```'
+        else:
+            print("Error! Must supply either a valid script name or a raw PowerShell command to execute.")
+            return False
+
+        payload = {
+            'base_command': 'runscript',
+            'session_id': session_id,
+            'command_string': cmd_string
+        }
+
+        response = self.session.post(self.base_url + '/entities/admin-command/v1', json=payload)
+
+        return response.status_code == 201
+
     def session_file_list(self, session_id: str) -> dict:
         """
         Get a list of files for the specified session.
@@ -802,7 +830,7 @@ class RTRClient:
 
     def get_scripts(self) -> List:
         """
-        Get  alist of custom-script ID's available for the user for the 'runscript' command.
+        Get a list of custom-script ID's available for the user for the 'runscript' command.
         :return:
         """
 
@@ -900,6 +928,41 @@ class RTRClient:
 
         return response.status_code == 200
 
+    def batch_runscript(self, batch_id: str, script_name: str = None, raw_cmd: str = None,
+                        optional_hosts: List = None) -> bool:
+        """
+        Execute either a pre-staged PowerShell script or supply a raw PowerShell command. Added this specifically
+        because the format for the runscript command can be tricky and is a pain to debug.
+        :param batch_id: Batch ID retrieved from the batch_init() method.
+        :param script_name: Name of a pre-staged script.
+        :param raw_cmd: Valid PowerShell command to execute across hosts in batch session.
+        :param optional_hosts: Any hosts you'd like excluded from command execution.
+        :return:
+        """
+        true = True
+
+        if script_name:
+            cmd_string = f'runscript -CloudFile="{script_name}"'
+        elif raw_cmd:
+            cmd_string = f'runscript -Raw=```{raw_cmd}```'
+        else:
+            print("Error! Must supply either a valid script name or a raw PowerShell command to execute.")
+            return False
+
+        payload = {
+            'base_command': 'runscript',
+            'batch_id': batch_id,
+            'command_string': cmd_string,
+            'persist_all': true
+        }
+
+        if optional_hosts:
+            payload['optional_hosts'] = optional_hosts
+
+        response = self.session.post(self.base_url + '/combined/batch-admin-command/v1', json=payload)
+
+        return response.status_code == 201
+
     def file_list(self) -> dict:
         """
         Get a list of put-file ID's that are available to the user for the 'put' command.
